<?php

namespace app\event\order;

use app\admin\service\coupon\CouponService;
use app\admin\service\order\CommissionService;
use app\model\Member;
use app\model\MemberAccount;
use app\model\PayDetail;
use app\model\SysConfig;
use support\Db;
use support\exception\BusinessException;
use support\Log;

class PartnerEvent
{
    public function grant($params)
    {
        Db::beginTransaction();
        try {
            Log::info("进入会员合伙人事件", $params);
            $config = SysConfig::where('config_key', 'app-identity-partner-config')->first();
            $configParams = json_decode($config->config_value_json, true);
            // 更新身份
            Member::where('member_id', $params['member_id'])->update(['member_is_partner' => 'Y']);
            // 余额账户
            $memberAccount = MemberAccount::where('join_account_member_id', $params['member_id'])->where('member_account_classify', 'CASH')->first();
            // 分期付款
            if (isset($params['order_amount_paid']) && $params['order_amount_paid'] > 0) {
                $params['order_amount_pay'] = $params['order_amount_paid'] + $params['order_amount_pay'];
            }
            // 增值到最后一条支付记录
            $added = 0;
            $payDetail = PayDetail::whereJsonContains('join_pay_object_json->order_id', $params['orderId'])->orderBy('pay_addtimes', 'DESC')->first();
            if (!empty($configParams['account']['added'])) {
                $params['order_amount_pay'] = 10000;
                $added = round($params['order_amount_pay'] * $configParams['account']['added'], 2);
                $payDetail->pay_extend_json = json_encode(['added_rate' => $configParams['account']['added'], 'added_amount' => $added]);
                $payDetail->save();
            }
            // 入账
            $memberAccount->member_account_income = $memberAccount->member_account_income + $params['order_amount_pay'];
            $memberAccount->member_account_surplus = $memberAccount->member_account_surplus + $params['order_amount_pay'];
            $memberAccount->member_account_added = $memberAccount->member_account_added + $added;
            $memberAccount->save();

            // 给上级提成
            CommissionService::commission($configParams, $params['member_id'], $params['orderId'], $params['order_amount_pay'], '推荐会员合伙人');

            //发放优惠券
            CouponService::autoSendCouponByGoods($params);

            Db::commit();
        } catch (BusinessException $e) {
            Db::rollBack();
            _syslog('会员合伙人', '会员合伙人处理失败：' . $e->getMessage());
        } catch (\Exception $e) {
            Db::rollBack();
            Log::info('PARTNER_DISPOSE_ERROR：' . $e->getMessage(), $params);

            _syslog('会员合伙人', '会员合伙人处理失败');
        }
    }
}